//  /**********************************************************************
//  * This source code is public domain, and may be freely distributed,   *
//  * modified, and used for any purpose.  IT COMES WITH ABSOLUTELY NO    *
//  * WARRANTY OF ANY KIND.                                               *
//  **********************************************************************/

import java.io.*;
import java.text.DecimalFormat;
import com.daylight.Smiles;
import com.daylight.Finger;

class calcfp extends Smiles {
    public static void main(String[] args) throws IOException {
	BufferedReader in = new BufferedReader
            (new InputStreamReader(System.in));
	PrintWriter out = new PrintWriter(System.out, true);
	String line, tdt;
	int molecule, fingerprint, ascii, osiz, ocnt, nsiz, ncnt;
	byte[] fstr, astr;

	// arguments
	if (0 < args.length) {
	    // help
	    if (args[0].startsWith("-h")) {
		System.err.println("Usage: java " +
				   new calcfp().getClass().getName() +
				   " < input.smi");
		System.exit(0);
	    }
	}

        // loop over input (read SMILES)
        while (null != (line=in.readLine())) {
            // get molecule
            if (NULL_OB == (molecule = Smiles.dt_smilin(line))) {
                System.err.println("WARNING: invalid SMILES (" + line + ")");
                continue;
            }

	    // make fingerprint
            if (NULL_OB == (fingerprint =
			    Finger.dt_fp_generatefp(molecule, 0, 7, 2048))) {
                System.err.println("ERROR: dt_fp_generatefp failed ("
				   + line + ")");
            }

	    // convert to ASCII
	    if (null == (fstr = Smiles.dt_stringvalue(fingerprint))){
                System.err.println("ERROR: FP string value (" + line + ")");
		System.exit(1);
	    }
	    if (NULL_OB == (ascii = Smiles.dt_binary2ascii(fstr))) {
                System.err.println("ERROR: binary2ascii (" + fstr + ")");
		System.exit(1);
	    }
	    if (null == (astr = Smiles.dt_stringvalue(ascii))) {
                System.err.println("ERROR: ASCII string value (" + fstr + ")");
		System.exit(1);
	    }

	    // show TDT - SMILES and 2D coordinates
	    // molecule or reaction
	    if (TYP_REACTION == Smiles.dt_type(molecule))
		tdt = "$SMI<\"" + line + "\">";
	    else
		tdt = "$SMI<" + line + ">";
	    // fingerprint
	    tdt = tdt + "FP<" + new String(astr);
	    osiz = Finger.dt_fp_obits(fingerprint);
	    ocnt = Finger.dt_fp_obitcount(fingerprint);
	    nsiz = Finger.dt_fp_nbits(fingerprint);
	    ncnt  = Finger.dt_fp_bitcount(fingerprint);
	    tdt = tdt + ";" + osiz + ";" + ocnt + ";" + nsiz + ";" + ncnt + ";;>|";
	    // show TDT
	    out.println(tdt);

	    // deallocate
	    Smiles.dt_dealloc(ascii);
	    Smiles.dt_dealloc(fingerprint);
	    Smiles.dt_dealloc(molecule);
	}

	// close I/O
        out.close();
   }
}
