    /**********************************************************************
    * This source code is public domain, and may be freely distributed,   *
    * modified, and used for any purpose.  IT COMES WITH ABSOLUTELY NO    *
    * WARRANTY OF ANY KIND.                                               *
    **********************************************************************/

#include <stdio.h>
#include <string.h>
#include <jni.h>
#include <dt_smiles.h>
#include <dt_depict.h>
#include "djutils.h"
#include "djSmiles.h"

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1abort (
    JNIEnv *env, jclass cls, jint ob){
    return dt_abort(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1add (
    JNIEnv *env, jclass cls, jint ob, jint elt) {
    return dt_add(ob, elt);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1addatom (
    JNIEnv *env, jclass cls, jint ob, jint atno, jint hct) {
    return dt_addatom(ob, atno, hct);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1addbond (
    JNIEnv *env, jclass cls, jint a0, jint a1, jint btyp) {
    return dt_addbond(a0, a1, btyp);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1addcomponent (
    JNIEnv *env, jclass cls, jint rxn, jint mol, jint role) {
   return dt_addcomponent(rxn, mol, role);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1adjunct (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_adjunct(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1alloc_1integer (
    JNIEnv *env, jclass cls) {
    return dt_alloc_integer();
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1alloc_1mol (
    JNIEnv *env, jclass cls) {
    return dt_alloc_mol();
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1alloc_1path (
    JNIEnv *env, jclass cls, jint mh) {
    return dt_alloc_path(mh);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1alloc_1reaction (
    JNIEnv *env, jclass cls) {
    return dt_alloc_reaction();
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1alloc_1real (
    JNIEnv *env, jclass cls) {
    return dt_alloc_real();
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1alloc_1seq (
    JNIEnv *env, jclass cls) {
    return dt_alloc_seq();
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1alloc_1string (
    JNIEnv *env, jclass cls, jstring s) {
    jint jret;
    char *cptr;
    int  clen;

    cptr = GetCharPtrLen(env, &clen, s);
    jret = dt_alloc_string(clen, cptr);
    if (cptr) ReleaseCharPtr(env, s, cptr);
    return jret;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1alloc_1substruct (
    JNIEnv *env, jclass cls, jint mh) {
    return dt_alloc_substruct(mh);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1append (
    JNIEnv *env, jclass cls, jint ob, jint elt) {
    return dt_append(ob, elt);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1appendstring (
    JNIEnv *env, jclass cls, jint ob, jstring name, jstring value) {
    jboolean jret;
    char *nptr, *vptr;
    int   nlen,  vlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    vptr = GetCharPtrLen(env, &vlen, value);
    jret = dt_appendstring(ob, nlen, nptr, vlen, vptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    if (vptr) ReleaseCharPtr(env, value, vptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1appendstringvalue (
    JNIEnv *env, jclass cls, jint ob, jbyteArray value) {
    jboolean jret;
    char *vptr;
    int   vlen;

    vptr = GetBinPtrLen(env, &vlen, value);
    jret = dt_appendstringvalue(ob, vlen, vptr);
    if (vptr) ReleaseBinPtr(env, value, vptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1arborder (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_arborder(ob);
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1arbsmiles (
    JNIEnv *env, jclass cls, jint ob, jboolean iso) {
    char *jret;
    int   jlen;

    jret = dt_arbsmiles(&jlen, ob, (iso==JNI_TRUE));
    return String2Java(env, jlen, jret);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1arbstream (
    JNIEnv *env, jclass cls, jint mh, jint type, jboolean iso, jboolean addh) {
    return dt_arbstream(mh, type, (iso==JNI_TRUE), (addh==JNI_TRUE));
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1aromatic (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_aromatic(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1ascii2binary (
    JNIEnv *env, jclass cls, jstring ascii) {
    jint jret;
    char *aptr;
    int   alen;

    aptr = GetCharPtrLen(env, &alen, ascii);
    jret = dt_ascii2binary(alen, aptr);
    if (aptr) ReleaseCharPtr(env, ascii, aptr);
    return jret;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1atend (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_atend(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1atstart (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_atstart(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1base (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_base(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1binary2ascii (
    JNIEnv *env, jclass cls, jbyteArray value) {
    jint jret;
    char *bptr;
    int   blen;

    bptr = GetBinPtrLen(env, &blen, value);
    jret = dt_binary2ascii(blen, bptr);
    if (bptr) ReleaseBinPtr(env, value, bptr);
    return jret;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1binary2asciilen (
    JNIEnv *env, jclass cls, jstring bin) {
    jint jret;
    char *bptr;
    int   blen;

    bptr = GetCharPtrLen(env, &blen, bin);
    jret = dt_binary2asciilen(blen, bptr);
    if (bptr) ReleaseCharPtr(env, bin, bptr);
    return jret;

}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1bond (
    JNIEnv *env, jclass cls, jint a0, jint a1) {
    return dt_bond(a0, a1);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1bondorder (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_bondorder(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1bondtype (
    JNIEnv *env, jclass cls, jint ob) {
   return dt_bondtype(ob);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1boolean (
    JNIEnv *env, jclass cls, jint ob, jstring name) {
    jboolean jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_boolean(ob, nlen, nptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1briefname (
    JNIEnv *env, jclass cls, jint ob) {
    char *jret;
    int   jlen;

    jret = dt_briefname(&jlen, ob);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1canatom_1stream (
    JNIEnv *env, jclass cls, jint mh, jboolean iso, jboolean addh) {
    return dt_canatom_stream(mh, (iso==JNI_TRUE), (addh==JNI_TRUE));
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1canbond_1stream (
    JNIEnv *env, jclass cls, jint mh, jboolean iso, jboolean addh) {
    return dt_canbond_stream(mh, (iso==JNI_TRUE), (addh==JNI_TRUE));
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1cansmi_1stream (
    JNIEnv *env, jclass cls, jint mh, jboolean iso) {
    return dt_cansmi_stream(mh, (iso==JNI_TRUE));
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1canstream (
    JNIEnv *env, jclass cls, jint mh, jint type, jboolean iso, jboolean addh) {
    return dt_canstream(mh, type, (iso==JNI_TRUE), (addh==JNI_TRUE));
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1cansmiles (
    JNIEnv *env, jclass cls, jint ob, jboolean iso) {
    char *jret;
    int   jlen;

    jret = dt_cansmiles(&jlen, ob, (iso==JNI_TRUE));
    return String2Java(env, jlen, jret);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1charge (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_charge(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1check_1license (
    JNIEnv *env, jclass cls, jint check_what, jstring product) {
    jint jret;
    char *pptr;
    int   plen;

    pptr = GetCharPtrLen(env, &plen, product);
    jret = dt_check_license(check_what, plen, pptr);
    if (pptr) ReleaseCharPtr(env, product, pptr);
    return jret;

}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1chiclass (
    JNIEnv *env, jclass cls, jint chival) {
    return dt_chiclass(chival);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1chiorder (
    JNIEnv *env, jclass cls, jint chival) {
    return dt_chiorder(chival);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1chiperm (
    JNIEnv *env, jclass cls, jint bseqh, jint bh, jint chival) {
    return dt_chiperm(bseqh, bh, chival);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1chiseq (
    JNIEnv *env, jclass cls, jint a, jint v) {
    return dt_chiseq(a, v);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1chival (
    JNIEnv *env, jclass cls, jint a, jint s) {
    return dt_chival(a, s);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1continue (
    JNIEnv *env, jclass cls, jint s, jintArray return_status) {
    jint jret;
    int   *stat;
    int    slen, sval;

    stat = GetIntPtrLen(env, &slen, return_status);
    slen && (sval = *stat);
    jret = dt_continue(s, &sval);
    slen && (*stat = sval);
    if (stat) ReleaseIntPtr(env, return_status, stat);
    return jret;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1copy (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_copy(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1count (
    JNIEnv *env, jclass cls, jint ob, jint typ) {
    return dt_count(ob, typ);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1database (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_database(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1datatype (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_datatype(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1dbo (
    JNIEnv *env, jclass cls, jint ob, jint b0, jint b1) {
    return dt_dbo(ob, b0, b1);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1dealloc (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_dealloc(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1delete (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_delete(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1description (
    JNIEnv *env, jclass cls, jint ob) {
    char *jret;
    int   jlen;

    jret = dt_description(&jlen, ob);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1dfnorm (
      JNIEnv *env, jclass cls, jint ob, jint norm) {
     return dt_dfnorm(ob, norm) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1dfnormdata (
    JNIEnv *env, jclass cls, jint ob, jint norm) {
    char *jret;
    int   jlen;

    jret = dt_dfnormdata(&jlen, ob, norm);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1done_1when (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_done_when(ob);
}

JNIEXPORT void JNICALL Java_com_daylight_Smiles_jdt_1errorclear (
    JNIEnv *env, jclass cls) {
    dt_errorclear();
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1errorworst (
    JNIEnv *env, jclass cls) {
    return dt_errorworst();
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1errors (
    JNIEnv *env, jclass cls, jint lev) {
    return dt_errors(lev);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1errorsave (
    JNIEnv *env, jclass cls, jstring func, jint lev, jstring msg) {
    jboolean jret;
    char *fptr, *mptr;

    fptr = GetCharPtr(env, func);
    mptr = GetCharPtr(env, msg);
    jret = dt_errorsave(fptr, lev, mptr);
    if (fptr) ReleaseCharPtr(env, func, fptr);
    if (mptr) ReleaseCharPtr(env, msg, mptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1evict (
    JNIEnv *env, jclass cls, jint ob, jstring password, jstring username, jint message, jboolean do_eviction) {
    jint jret;
    char *pptr, *uptr;
    int   plen,  ulen;

    pptr = GetCharPtrLen(env, &plen, password);
    uptr = GetCharPtrLen(env, &ulen, username);
    jret = dt_evict(ob, plen, pptr, ulen, uptr, message, (do_eviction==JNI_TRUE));

    if (pptr) ReleaseCharPtr(env, password, pptr);
    if (uptr) ReleaseCharPtr(env, username, uptr);
    return jret;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1exists (
    JNIEnv *env, jclass cls, jint server, jstring name) {
    jboolean jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_exists(server, nlen, nptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1fieldtype (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_fieldtype(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1getdatabases (
    JNIEnv *env, jclass cls, jint server) {
    return dt_getdatabases(server);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1getdatatype (
    JNIEnv *env, jclass cls, jint ob, jstring tag) {
    jint jret;
    char *tptr;
    int   tlen;

    tptr = GetCharPtrLen(env, &tlen, tag);
    jret = dt_getdatatype(ob, tlen, tptr);
    if (tptr) ReleaseCharPtr(env, tag, tptr);
    return jret;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1getmap (
    JNIEnv *env, jclass cls, jint atom) {
    return dt_getmap(atom);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1getpasswords (
    JNIEnv *env, jclass cls, jint server) {
    return dt_getpasswords(server);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1getrole (
    JNIEnv *env, jclass cls, jint ob1, jint ob2) {
    return dt_getrole(ob1, ob2);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1getsearchpath (
    JNIEnv *env, jclass cls, jint server) {
    return dt_getsearchpath(server);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1getusers (
    JNIEnv *env, jclass cls, jint server) {
    return dt_getusers(server);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1handle (
    JNIEnv *env, jclass cls, jint ob, jstring name) {
    jint jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_handle(ob, nlen, nptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1hcount (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_hcount(ob);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1hold (
    JNIEnv *env, jclass cls, jint ob, jstring pw) {
    jboolean jret;
    char *pptr;
    int   plen;

    pptr = GetCharPtrLen(env, &plen, pw);
    jret = dt_hold(ob, plen, pptr);
    if (pptr) ReleaseCharPtr(env, pw, pptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1imp_1hcount (
    JNIEnv *env, jclass cls, jint ob) {
   return dt_imp_hcount(ob);
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1info (
    JNIEnv *env, jclass cls, jint ob, jstring req) {
    char *jret, *rptr;
    int   jlen;

    rptr = GetCharPtr(env, req);
    jret = dt_info(&jlen, ob, rptr);
    if (rptr) ReleaseCharPtr(env, req, rptr);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1insert (
    JNIEnv *env, jclass cls, jint ob, jint elt) {
    return dt_insert(ob, elt);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1integer (
    JNIEnv *env, jclass cls, jint ob, jstring name) {
    jint jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_integer(ob, nlen, nptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1integervalue (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_integervalue(ob);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1invalid (
    JNIEnv *env, jclass cls, jint ob) {
   return dt_invalid(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1isheld (
    JNIEnv *env, jclass cls, jint ob) {
   return dt_isheld(ob) ? JNI_TRUE : JNI_FALSE;

}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1isopen (
    JNIEnv *env, jclass cls, jint server, jstring name) {
    jboolean jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_isopen(server, nlen, nptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1ispublic (
    JNIEnv *env, jclass cls, jint server, jstring name) {
    jboolean jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_ispublic(server, nlen, nptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1isohydro (
    JNIEnv *env, jclass cls) {
    return dt_isohydro();
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1mapped (
    JNIEnv *env, jclass cls, jint a1, jint a2) {
    return dt_mapped(a1, a2) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1member (
    JNIEnv *env, jclass cls, jint ob, jint elt) {
    return dt_member(ob, elt) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1mod_1is_1on (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_mod_is_on(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1mod_1off (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_mod_off(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1mod_1on (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_mod_on(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1molecule (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_molecule(ob);
}

/* v4.9
JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1molgraph (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_molgraph(ob);
}
*/

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1name (
    JNIEnv *env, jclass cls, jint ob) {
    char *jret;
    int   jlen;

    jret = dt_name(&jlen, ob);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1next (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_next(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1number (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_number(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1open (
    JNIEnv *env, jclass cls, jint s, jstring path,
    jstring perm, jstring pass, jintArray isnew) {
    jint jret;
    char *aptr, *eptr, *pptr;
    int   alen,  elen,  plen, *iptr, ilen, ival;

    aptr = GetCharPtrLen(env, &alen, path);
    eptr = GetCharPtrLen(env, &elen, perm);
    pptr = GetCharPtrLen(env, &plen, pass);
    iptr = GetIntPtrLen(env, &ilen, isnew);
    ilen && (ival = *iptr);
    jret = dt_open(s, alen, aptr, elen, eptr, plen, pptr, &ival);
    if (aptr) ReleaseCharPtr(env, path, aptr);
    if (eptr) ReleaseCharPtr(env, perm, eptr);
    if (pptr) ReleaseCharPtr(env, pass, pptr);
    ilen && (*iptr = ival);
    if (iptr) ReleaseIntPtr(env, isnew, iptr);
    return jret;

}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1origsmi_1stream (
    JNIEnv *env, jclass cls, jint mh) {
    return dt_origsmi_stream(mh);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1origstream (
    JNIEnv *env, jclass cls, jint mh, jint type) {
    return dt_origstream(mh, type);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1parent (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_parent(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1ping (
    JNIEnv *env, jclass cls, jint server, jstring s) {
    jint jret;
    char *cptr;
    int   clen;

    cptr = GetCharPtrLen(env, &clen, s);
    jret = dt_ping(server, clen, cptr);
    if (cptr) ReleaseCharPtr(env, s, cptr);
    return jret;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1propnames (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_propnames(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1proptype (
    JNIEnv *env, jclass cls, jint ob, jstring name) {
    jint jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_proptype(ob, nlen, nptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret;
}

JNIEXPORT jfloat JNICALL Java_com_daylight_Smiles_jdt_1real (
    JNIEnv *env, jclass cls, jint ob, jstring name) {
    jfloat jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_real(ob, nlen, nptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret;
}

JNIEXPORT jfloat JNICALL Java_com_daylight_Smiles_jdt_1realvalue (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_realvalue(ob);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1release (
    JNIEnv *env, jclass cls, jint ob, jstring pw) {
    jboolean jret;
    char *pptr;
    int   plen;

    pptr = GetCharPtrLen(env, &plen, pw);
    jret = dt_release(ob, plen, pptr);
    if (pptr) ReleaseCharPtr(env, pw, pptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1remove (
    JNIEnv *env, jclass cls, jint ob, jint elt) {
    return dt_remove(ob, elt) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1reset (
      JNIEnv *env, jclass cls, jint ob) {
    return dt_reset(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1seqappend (
    JNIEnv *env, jclass cls, jint ob, jint elt) {
    return dt_seqappend(ob, elt);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1seqdelete (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_seqdelete(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1seqinsert (
    JNIEnv *env, jclass cls, jint ob, jint elt) {
    return dt_seqinsert(ob, elt);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1seqsort (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_seqsort(ob) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1server (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_server(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1set_1server_1timeout (
    JNIEnv *env, jclass cls, jint resw_new) {
    return dt_set_server_timeout(resw_new);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1setadjunct (
    JNIEnv *env, jclass cls, jint ob, jint adj) {
    return dt_setadjunct(ob, adj);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setarborder (
    JNIEnv *env, jclass cls, jint ob, jint val) {
    return dt_setarborder(ob, val) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setaromatic (
    JNIEnv *env, jclass cls, jint ob, jboolean val) {
    return dt_setaromatic(ob, (val==JNI_TRUE)) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setboolean (
    JNIEnv *env, jclass cls, jint ob, jstring name,
    jboolean value) {
    jboolean jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_setboolean(ob, nlen, nptr, (value==JNI_TRUE));
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setbondorder (
    JNIEnv *env, jclass cls, jint ob, jint val) {
    return dt_setbondorder(ob, val) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setbondtype (
    JNIEnv *env, jclass cls, jint ob, jint val) {
    return dt_setbondtype(ob, val) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setcharge (
    JNIEnv *env, jclass cls, jint ob, jint val) {
   return dt_setcharge(ob, val) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setchival (
    JNIEnv *env, jclass cls, jint a, jint s, jint v) {
    return dt_setchival(a, s, v) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setdbo (
    JNIEnv *env, jclass cls, jint ob, jint b0, jint b1, jint v) {
   return dt_setdbo(ob, b0, b1, v) ? JNI_TRUE : JNI_FALSE;

}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1sethandle (
    JNIEnv *env, jclass cls, jint ob, jstring name, jint value) {
    jboolean jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_sethandle(ob, nlen, nptr, value);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setimp_1hcount (
    JNIEnv *env, jclass cls, jint ob, jint val) {
    return dt_setimp_hcount(ob, val) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setinteger (
    JNIEnv *env, jclass cls, jint ob, jstring name, jint value) {
    jboolean jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_setinteger(ob, nlen, nptr, value);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setintegervalue (
    JNIEnv *env, jclass cls, jint ob, jint value) {
   return dt_setintegervalue(ob, value) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setmap (
    JNIEnv *env, jclass cls, jint a1, jint a2) {
   return dt_setmap(a1, a2) ? JNI_TRUE : JNI_FALSE;

}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setmessage (
    JNIEnv *env, jclass cls, jint ob, jstring password,
    jint when, jint message) {
    jboolean jret;
    char *pptr;
    int   plen;

    pptr = GetCharPtrLen(env, &plen, password);
    jret = dt_setmessage(ob, plen, pptr, when, message);
    if (pptr) ReleaseCharPtr(env, password, pptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setnumber (
    JNIEnv *env, jclass cls, jint ob, jint val) {
    return dt_setnumber(ob, val) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setpassword (
    JNIEnv *env, jclass cls, jint ob, jstring what, jstring oldpw,
    jstring newpw) {
    jboolean jret;
    char *wptr, *optr, *nptr;
    int   wlen,  olen,  nlen;

    wptr = GetCharPtrLen(env, &wlen, what);
    optr = GetCharPtrLen(env, &olen, oldpw);
    nptr = GetCharPtrLen(env, &nlen, newpw);
    jret = dt_setpassword(ob, wptr, optr, nptr);
    if (wptr) ReleaseCharPtr(env, what,  wptr);
    if (optr) ReleaseCharPtr(env, oldpw, optr);
    if (nptr) ReleaseCharPtr(env, newpw, nptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setreal (
    JNIEnv *env, jclass cls, jint ob, jstring name, jfloat value) {
    jboolean jret;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_setreal(ob, nlen, nptr, value);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setrealvalue (
    JNIEnv *env, jclass cls, jint ob, jfloat value) {
    return dt_setrealvalue(ob, value) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setsearchpath (
    JNIEnv *env, jclass cls, jint server, jstring password,
    jstring dir, jint replace) {
    jboolean jret;
    char *pptr;
    int   plen;
    char *dptr;
    int   dlen;

    pptr = GetCharPtrLen(env, &plen, password);
    dptr = GetCharPtrLen(env, &dlen, dir);
    jret = dt_setsearchpath(server, plen, pptr, dlen, dptr, replace);    
    if (pptr) ReleaseCharPtr(env, password, pptr);
    if (dptr) ReleaseCharPtr(env, dir, dptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setstring (
    JNIEnv *env, jclass cls, jint ob, jstring name, jstring value) {
    jboolean jret;
    char *nptr, *vptr;
    int   nlen,  vlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    vptr = GetCharPtrLen(env, &vlen, value);
    jret = dt_setstring(ob, nlen, nptr, vlen, vptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    if (vptr) ReleaseCharPtr(env, value, vptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setstringvalue (
    JNIEnv *env, jclass cls, jint ob, jbyteArray value) {
    jboolean jret;
    char *vptr;
    int   vlen;

    vptr = GetBinPtrLen(env, &vlen, value);
    jret = dt_setstringvalue(ob, vlen, vptr);
    if (vptr) ReleaseBinPtr(env, value, vptr);
    return jret ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1setweight (
    JNIEnv *env, jclass cls, jint ob, jint val) {
   return dt_setweight(ob, val) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1smilin (
    JNIEnv *env, jclass cls, jstring s) {
    jint jret;
    char *cptr;
    int  clen;

    cptr = GetCharPtrLen(env, &clen, s);
    jret = dt_smilin(clen, cptr);
    if (cptr) ReleaseCharPtr(env, s, cptr);
    return jret;
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1smilinerrors (
    JNIEnv *env, jclass cls) {
    return dt_smilinerrors();
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1smilinerrtext (
    JNIEnv *env, jclass cls) {
    char *jret;
    int   jlen;

    jret = dt_smilinerrtext(&jlen);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1stream (
    JNIEnv *env, jclass cls, jint ob, jint typ) {
    return dt_stream(ob, typ);
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1string (
    JNIEnv *env, jclass cls, jint ob, jstring name) {
    char *jret;
    int   jlen;
    char *nptr;
    int   nlen;

    nptr = GetCharPtrLen(env, &nlen, name);
    jret = dt_string(&jlen, ob, nlen, nptr);
    if (nptr) ReleaseCharPtr(env, name, nptr);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jbyteArray JNICALL Java_com_daylight_Smiles_jdt_1stringvalue (
    JNIEnv *env, jclass cls, jint ob) {
    char *jret;
    int   jlen;

    jret = dt_stringvalue(&jlen, ob);
    return Binary2Java(env, jlen, jret);
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1summary (
    JNIEnv *env, jclass cls, jint ob) {
    char *jret;
    int   jlen;

    jret = dt_summary(&jlen, ob);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1symbol (
    JNIEnv *env, jclass cls, jint ob) {
    char *jret;
    int   jlen;

    jret = dt_symbol(&jlen, ob);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1symclass (
    JNIEnv *env, jclass cls, jint ob, jboolean i) {
    return dt_symclass(ob, (i==JNI_TRUE));
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1symorder (
    JNIEnv *env, jclass cls, jint ob, jboolean i) {
    return dt_symorder(ob, (i==JNI_TRUE));
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1tag (
    JNIEnv *env, jclass cls, jint ob) {
    char *jret;
    int   jlen;

    jret = dt_tag(&jlen, ob);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jboolean JNICALL Java_com_daylight_Smiles_jdt_1toend (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_toend(ob) ? JNI_TRUE : JNI_FALSE;

}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1type (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_type(ob);
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1typename (
    JNIEnv *env, jclass cls, jint ob) {
    char *jret;
    int   jlen;

    jret = dt_typename(&jlen, ob);
    return String2Java(env, jlen, jret);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1uid (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_uid(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1uidrange (
    JNIEnv *env, jclass cls, jint ob, jint typ) {
    return dt_uidrange(ob, typ);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1weight (
    JNIEnv *env, jclass cls, jint ob) {
    return dt_weight(ob);
}

JNIEXPORT jint JNICALL Java_com_daylight_Smiles_jdt_1xatom (
    JNIEnv *env, jclass cls, jint a, jint b) {
    return dt_xatom(a, b);
}

JNIEXPORT jstring JNICALL Java_com_daylight_Smiles_jdt_1xsmiles (
    JNIEnv *env, jclass cls, jint ob, jboolean iso, 
    jboolean explicit) {
    char *jret;
    int   jlen;

    jret = dt_xsmiles(&jlen, ob, (iso==JNI_TRUE), (explicit==JNI_TRUE));
    return String2Java(env, jlen, jret);
}
