    /**********************************************************************
    * This source code is public domain, and may be freely distributed,   *
    * modified, and used for any purpose.  IT COMES WITH ABSOLUTELY NO    *
    * WARRANTY OF ANY KIND.                                               *
    **********************************************************************/

#include <stdio.h>
#include <jni.h>

/* Java & C datatype mapping */

/* binary */
char* GetBinPtr     (JNIEnv *env, jbyteArray jbyt) {
    return (char*)(*env)->GetByteArrayElements(env, jbyt, NULL);
}
char* GetBinPtrLen  (JNIEnv *env, int *len, jbyteArray jbyt) {
    *len = (int)(*env)->GetArrayLength(env, jbyt);
    return GetBinPtr(env, jbyt);
}
void   ReleaseBinPtr (JNIEnv *env, jbyteArray jbyt, char *jptr) {
    (*env)->ReleaseByteArrayElements(env, jbyt, (jbyte*)jptr, 0);
}

/* characters */
char* GetCharPtr     (JNIEnv *env, jstring jstr) {
    return (char*)(*env)->GetStringUTFChars(env, jstr, NULL);
}
char* GetCharPtrLen  (JNIEnv *env, int *len, jstring jstr) {
    *len = (int)(*env)->GetStringLength(env, jstr);
    return GetCharPtr(env, jstr);
}
void  ReleaseCharPtr (JNIEnv *env, jstring jstr, char *jptr) {
    (*env)->ReleaseStringUTFChars(env, jstr, (const char*)jptr);
}

/* floating point */
float* GetFloatPtr     (JNIEnv *env, jfloatArray jfarray) {
    return (float*)(*env)->GetFloatArrayElements(env, jfarray, NULL );
}
float* GetFloatPtrLen  (JNIEnv *env, int *len, jfloatArray jfarray) {
    *len = (int)(*env)->GetArrayLength(env, jfarray);
    return GetFloatPtr(env, jfarray);
}
void    ReleaseFloatPtr (JNIEnv *env, jfloatArray jfarray, float* fptr) {
   (*env)->ReleaseFloatArrayElements(env, jfarray, (jfloat*)fptr, 0);
}

/* integer */
int* GetIntPtr    (JNIEnv *env, jintArray jiarray) {
    return (int*)(*env)->GetIntArrayElements(env, jiarray, NULL);
}
int* GetIntPtrLen (JNIEnv *env, int *len, jintArray jiarray) {
    *len = (int)(*env)->GetArrayLength(env, jiarray);
    return GetIntPtr(env, jiarray);
}
void ReleaseIntPtr (JNIEnv *env, jintArray jiarray, int *jptr) {
    (*env)->ReleaseIntArrayElements(env, jiarray, (jint*)jptr, 0);
}

/* data type converter */
jbyteArray Binary2Java (JNIEnv *env, int len, char *str) {
    jbyteArray  jbyt;
    char *jptr;

    if (NULL == str) return NULL;
    if (NULL != (jbyt = (*env)->NewByteArray(env, len))) {
	jptr = GetBinPtr(env, jbyt);
	memcpy((void*)jptr, str, len);
	ReleaseBinPtr(env, jbyt, jptr);
    }
    return jbyt;
}
jstring String2Java    (JNIEnv *env, int len, char *str) {
    jstring  jstr;
    char *jptr;

    if (NULL == str) return NULL;
    if (NULL != (jstr = (*env)->NewStringUTF(env, str))) {
	jptr = GetCharPtr(env, jstr);
	memcpy((void*)jptr, str, len);
	ReleaseCharPtr(env, jstr, jptr);
    }
    return jstr;
}
