//  /**********************************************************************
//  * This source code is public domain, and may be freely distributed,   *
//  * modified, and used for any purpose.  IT COMES WITH ABSOLUTELY NO    *
//  * WARRANTY OF ANY KIND.                                               *
//  **********************************************************************/

import java.io.*;
import com.daylight.Smiles;
import com.daylight.Smarts;

class smartsmatch extends Smiles {
    public static int stream2seq(int stream) {
	int ob, seq = Smiles.dt_alloc_seq();
	
	Smiles.dt_reset(stream);
	while (NULL_OB != (ob = Smiles.dt_next(stream)))
	    Smiles.dt_append(seq, ob);
	return seq;
    }


    public static boolean explicit_hydro(int molecule) {
	int atomstream, atomseq, atom, hyd;
	int imp_hcount;

	Smiles.dt_mod_on(molecule);
	atomstream = Smiles.dt_stream(molecule, TYP_ATOM);
	atomseq = stream2seq(atomstream);
	dt_dealloc(atomstream);
	while (NULL_OB != (atom = Smiles.dt_next(atomseq))) {
	    imp_hcount = Smiles.dt_imp_hcount(atom);
	    while (imp_hcount-- > 0) {
		hyd = Smiles.dt_addatom(molecule, DX_ATN_H, 0);
		Smiles.dt_addbond(atom, hyd, 1);
	    }
	    Smiles.dt_setimp_hcount(atom, 0);
	}

	return Smiles.dt_mod_off(molecule);
    }

    public static void main(String[] args) throws IOException {
	BufferedReader in = new BufferedReader
	    (new InputStreamReader(System.in));
        PrintWriter out = new PrintWriter(System.out, true);
	boolean help;
        String smarts, line;
        int pattern, molecule, pathset, paths, path;
	int i, count, smi_in, smi_out;
	boolean explicit_hs, match;

	// initialize
	smi_in = smi_out = 0;
	help = false;
	count = 1;
	explicit_hs = false;
	smarts = null;
	match = false;

	// arguments
	if (0 == args.length)
	    help = true;
	else {
	    for (i=0; i<args.length; i++) {
		// help
		if (args[i].startsWith("-h")) {
		    help = true;
		    break;
		}
		// SMARTS pattern
		else if (true == args[i].startsWith("-count")) {
		    if (++i <= args.length)
			count = Integer.parseInt(args[i]);
		    else {
			System.err.println("ERROR: missing count");
			System.exit(0);
		    }
		}
		else if (true == args[i].startsWith("-explicit_hydrogens"))
		    explicit_hs = true;
		else if (true == args[i].startsWith("-smarts")) {
		    if (++i < args.length)
			smarts = args[i];
		    else {
			System.err.println("ERROR: missing SMARTS");
			System.exit(0);
		    }
		}
		else
		    System.err.println("WARNING: unknown option (" +
				       args[i] + ")");

	    }
	}
	// help
	if (true == help || null == smarts) {
	    System.err.println("Usage: java " +
			       new smartsmatch().getClass().getName() +
			       " -smarts <pattern> [ -count N ] " +
			       "[ -explicit_htdrogens ] < input.smi");
	    System.exit(0);
	}


	// get pattern
	if (NULL_OB == (pattern = Smarts.dt_smartin(smarts))) {
	    System.err.println("ERROR: invalid SMARTS (" + smarts + ")");
	    System.exit(0);
	}

	// loop over input (read SMILES)
	out.println("Reading SMILES from stdin...");
        while (null != (line=in.readLine())) {
	    smi_in++;
	    match = false;

	    // get molecule
	    if (NULL_OB == (molecule = Smiles.dt_smilin(line))) {
		System.err.println("ERROR: invalid SMILES (" + line + ")");
		continue;
	    }

	    // get molecule
	    if (true == explicit_hs) {
		if (false == (explicit_hydro(molecule))) {
		    System.err.println("ERROR: can't make H's explicit (" +
				       line + ")");
		    continue;
		}
	    }

	    // get match
	    switch (count) {
	    case 0:
		pathset = Smarts.dt_match(pattern, molecule, 1);
		if (NULL_OB == pathset)
		    match = true;
		break;
	    case 1:
		pathset = Smarts.dt_match(pattern, molecule, 1);
		if (NULL_OB != pathset)
		    match = true;
		break;
	    default:
		pathset = Smarts.dt_umatch(pattern, molecule, 0);
		if (NULL_OB != pathset)
		    if (count <= Smiles.dt_count(pathset, TYP_PATH))
			match = true;
		break;
	    }

	    // SMILES output
	    if (true == match) {
		++smi_out;
		out.println(line);
	    }

	    // deallocate
	    Smiles.dt_dealloc(molecule);

	    // voluntarily relinquish processor
	    Thread.yield();
	}

	// show results
	out.println(new smartsmatch().getClass().getName() + " results: " +
		    smi_in + " SMILES in, " + smi_out + " SMILES out");

	// close I/O
        out.close();
        in.close();
    }
}
